@@ -196,6 +196,7 @@ mod test {
 		let proof = prove_groth16_x5::<_, Bls12_381>(&pk, circuit, rng);
 
 		let res = verify_groth16::<Bls12_381>(&vk, &public_inputs, &proof);
+		println!("{}", res);
 		assert!(res);
 	}
 
@@ -225,9 +226,11 @@ mod test {
 		let params5 = setup_params_x5_5(curve);
 		let relayer = BlsFr::rand(rng);
 		let recipient = BlsFr::rand(rng);
+		let fee = BlsFr::from(0);
+		let refund = BlsFr::from(0);
 		let (leaf_private, leaf, nullifier_hash) = setup_leaf_x5(&params5, rng);
 
-		let arbitrary_input = setup_arbitrary_data(recipient, relayer);
+		let arbitrary_input = setup_arbitrary_data(recipient, relayer, fee, refund);
 		let params3 = setup_params_x5_3(curve);
 		let (_, path) = setup_tree_and_create_path_x5(&[leaf], 0, &params3);
 		let root = BlsFr::rand(rng);
@@ -261,9 +264,11 @@ mod test {
 		let params5 = setup_params_x5_5(curve);
 		let relayer = BlsFr::rand(rng);
 		let recipient = BlsFr::rand(rng);
+		let fee = BlsFr::from(0);
+		let refund = BlsFr::from(0);
 		let (leaf_private, _, nullifier_hash) = setup_leaf_x5(&params5, rng);
 		let leaf = BlsFr::rand(rng);
-		let arbitrary_input = setup_arbitrary_data(recipient, relayer);
+		let arbitrary_input = setup_arbitrary_data(recipient, relayer, fee, refund);
 		let params3 = setup_params_x5_3(curve);
 		let (_, path) = setup_tree_and_create_path_x5(&[leaf], 0, &params3);
 		let root = BlsFr::rand(rng);
@@ -297,9 +302,11 @@ mod test {
 		let params5 = setup_params_x5_5(curve);
 		let relayer = BlsFr::rand(rng);
 		let recipient = BlsFr::rand(rng);
+		let fee = BlsFr::from(0);
+		let refund = BlsFr::from(0);
 		let (leaf_private, leaf, _) = setup_leaf_x5(&params5, rng);
 		let nullifier_hash = BlsFr::rand(rng);
-		let arbitrary_input = setup_arbitrary_data(recipient, relayer);
+		let arbitrary_input = setup_arbitrary_data(recipient, relayer, fee, refund);
 		let params3 = setup_params_x5_3(curve);
 		let (_, path) = setup_tree_and_create_path_x5(&[leaf], 0, &params3);
 		let root = BlsFr::rand(rng);
